/*
 * Copyright (c) 2005-2010 Imperas Software Ltd., www.imperas.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include "icm/icmCpuManager.h"

icmProcessorP createPlatform(void) {

////////////////////////////////////////////////////////////////////////////////

    icmInit(ICM_VERBOSE|ICM_STOP_ON_CTRLC, 0, 0);

    icmSetPlatformName("BareMetalPowerpc32Single");


////////////////////////////////////////////////////////////////////////////////
//                                 Bus bus1_b                                 //
////////////////////////////////////////////////////////////////////////////////

    icmBusP bus1_b = icmNewBus( "bus1_b", 32);


////////////////////////////////////////////////////////////////////////////////
//                               Processor cpu1                               //
////////////////////////////////////////////////////////////////////////////////
#define SIM_ATTRS (ICM_ATTR_RELAXED_SCHED)

    icmAttrListP cpu1_attr = icmNewAttrList();

    icmAddStringAttr(cpu1_attr, "compatibility", "isa");
    icmAddStringAttr(cpu1_attr, "endian",        "big");
    icmAddDoubleAttr(cpu1_attr, "mips", 100.000000);

    const char *powerpc32Model    = icmGetVlnvString(NULL, "power.ovpworld.org", "processor",  "powerpc32",        "1.0", "model");
    const char *powerpc32Semihost = icmGetVlnvString(NULL, "power.ovpworld.org", "semihosting", "powerpc32Newlib", "1.0", "model");

    icmProcessorP cpu1_c = icmNewProcessor(
        "cpu1",   // name
        "powerpc32",   // type
        0,   // cpuId
        0x0, // flags
         32,   // address bits
        powerpc32Model,   // model
        "modelAttrs",   // symbol
        SIM_ATTRS,   // procAttrs
        cpu1_attr,   // attrlist
        powerpc32Semihost,   // semihost file
        "modelAttrs"  // semihost symbol
    );

    icmConnectProcessorBusses( cpu1_c, bus1_b, bus1_b );


////////////////////////////////////////////////////////////////////////////////
//                                Memory memory                               //
////////////////////////////////////////////////////////////////////////////////

    icmMemoryP memory_m = icmNewMemory("memory_m", 0x7, 0xffffffff);

    icmConnectMemoryToBus( bus1_b, "sp1", memory_m, 0x0);

    return cpu1_c;
}

////////////////////////////////////////////////////////////////////////////////
//                                   M A I N                                  //
////////////////////////////////////////////////////////////////////////////////

//int main(int argc, char *argv[]) {
int main() {

    /*// Check arguments for application to load
    if(argc!=2) {
        // incorrect arguments
        icmPrintf("Usage : %s <application name>\n", argv[0]);
        return -1;
    }
    const char *appName = argv[1];*/
    const char *appName = "fibonacci.POWERPC32.elf";

    // the constructor
    icmProcessorP processor = createPlatform();

    // load the application executable file into processor memory space
    if(!icmLoadProcessorMemory(processor, appName, False, False, True)) {
        return -1;
    }

    // run till the end
    icmSimulatePlatform();

    // free simulation data structures
    icmTerminate();

    return 0;
}

////////////////////////////////////////////////////////////////////////////////
